@@ -1,6 +1,6 @@
 context("flag / L / F")
 
-rm(list = ls())
+# rm(list = ls())
 
 x <- abs(10*rnorm(100))
 xNA <- x
